#ifndef _MY_WIRELESS_CONTROLLER_H
#define _MY_WIRELESS_CONTROLLER_H

#define WIRELESS_CONTROLLER_DEBUG_MODE 0

#include <inttypes.h>
//#include "DigitalWriteFast.h"

#if ARDUINO >= 100
  #include "Arduino.h"
#else
  #include "WProgram.h"
#endif

#define ENCODING_PULSE_WIDTH 1
#define ENCODING_MANCHESTER 2

#define INPUT_ACTIVE_LOW 0
#define INPUT_ACTIVE_HIGH 1

#define RECEIVING_START 0
#define RECEIVING_BITS 1
#define RECEIVING_END 2
#define RECEIVING_MANCHESTER_END 3
#define ZERO_BIT 0
#define ONE_BIT  1
#define ANY_BIT  2
#define MARK_NO_MARK  100
#define MARK_ERROR    0

class WirelessController
{
public:
	// Constructor for the class. Receives all of the data describing
	// the protocol we want to send and/or receive
	WirelessController( unsigned long *marks_duration,
						int *message_code_start,
						int *message_code_one,
						int *message_code_zero,
						int *message_code_end,
						byte number_of_marks, 
						byte number_of_marks_start, 
						byte number_of_marks_bit, 
						byte number_of_marks_end,
						byte mark_tolerance,
						byte encoding_type,
						byte input_type,
						int number_of_bits);
		
	// Initializes the controller to receive commands
	void intializeReading(byte read_pin);
	// Initializes the controller to send commands
	void intializeWritting(byte write_pin, boolean modulate_output);
	
	// Sends the command in the message buffer
	void sendCode(byte *message_buffer);

	// Checks if a code has been received since the last check
	// This function needs to be called often... as in... REALLY REALLY often
	// All the measurements are done here, so the longer it takes to call it,
	// the higher is the probability of missing a reading
	boolean codeReceived();

	// Compares the given code with the last code received 
	boolean compareCode(byte* code);
	// Copies the last received code to a external buffer
	void getCode(byte* code);
	
	// Debug functions...
	void codeCheckMarks();
	
protected:
	// Hardware setup variables
	byte write_pin;
	boolean modulate_output;
	byte read_pin;
	
	// Used for the timing on the emission
	unsigned long end_last_code;
	// Used to keep track of data when receiving
	byte * code_buff; 
	byte bit_number, byte_value;
	byte mark_status;
	int last_input_value;
	unsigned long swap_time, last_swap_time;
	byte reception_status;
	byte reception_position;
	int * message_bit_marks_buffer;
	byte bit_value;
	
	// Auxiliary functions
	void sendMark(int code);
	boolean processIncomingBit(byte next_bit);
	void sendBits(byte *bit_buffer, int count);	
	int readMark();
	boolean codeReceivedPulseWidth();
	boolean codeReceivedManchester();
	
//  Debug stuff, to be removed
//	#define BUFFER_SIZE 5
//	byte buff_pos;
//	char charbuff[BUFFER_SIZE];
//	unsigned long intbuff[BUFFER_SIZE];
//	void buffPrint(char next_char);
		
	// Variables that store the data containing the code information
	// Duration of all the known marks in the code
	byte number_of_marks;
	unsigned long * marks_duration;
	// Marks that start the message
	byte number_of_marks_start;
	int * message_code_start;
	// Marks that represent the different bits
	byte number_of_marks_bit;
	int * message_code_one;
	int * message_code_zero;
	// Marks that end the message
	byte number_of_marks_end;
	int * message_code_end;
	// Tolerance to timing measurement errors
	byte mark_tolerance;
	// Type of encoding used: ENCODING_PULSE_WIDTH or ENCODING_MANCHESTER
	byte encoding_type;
	// Type of input: INPUT_ACTIVE_HIGH or INPUT_ACTIVE_LOW
	byte input_type;
	// Number of bits contained in the message
	int number_of_bits;
};

#endif

